package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.roundness;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesCustSoftInfoService;
import cn.estsh.i3plus.ext.mes.api.base.strategy.IRoundnessStrategy;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.busi.MesCreateWorkOrderImpl;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartProdGroup;
import cn.estsh.i3plus.pojo.mes.bean.MesPartProdGroupDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesPartSap;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesCustSortInfo;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesPartSapRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description : 按照包装圆整
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/2 21:22
 * @Modify:
 **/
@Service
public class MesPackageRoundnessService implements IRoundnessStrategy {

    private static final Logger log = LoggerFactory.getLogger(MesPackageRoundnessService.class);

    @Autowired
    private MesPartSapRepository mesPartSapRao;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private IMesCustSoftInfoService custSoftInfoService;

    @Autowired
    private MesCreateWorkOrderImpl mesCreateWorkOrder;

    @Autowired
    private IMesConfigService mesConfigService;

    @Override
    public void execute(MesPartProdGroup partProdGroup, List<MesCustSortInfo> sortInfoList, List<MesPartProdGroupDetail> details) {
        try {
            //是否是头部零件
            Integer isSeqInfoHeadPart = partProdGroup.getIsSeqInfoHeadPart();
            if (isSeqInfoHeadPart == null || isSeqInfoHeadPart == CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) {
                log.info("非头零件不能按照包装圆整;零件生产组为:{}", partProdGroup.getPartProdGroupCode());
                return;
            }

            Map<String, MesPartProdGroupDetail> prodGroupDetailMap = details.stream().collect(Collectors.toMap(MesPartProdGroupDetail::getCustPartNo, Function.identity()));
            //1.生产组中的圆整数量
            double roundQty = Double.parseDouble(partProdGroup.getRoundQty());
            //2.排序信息中的零件合计数量
            double qty = sortInfoList.stream().map(MesCustSortInfo::getQty).reduce(Double::sum).get();
            MesCustSortInfo sortInfo;
            /**
             * 2024/10/24 by Castle需要根据custOrderNo升序设置productSeq
             */
            log.info("castle-execute-before查询出的sortInfo数据:{}", sortInfoList);
            sortInfoList = sortInfoList.stream().sorted((sortInfo1, sortInfo2) -> {
                if (Long.parseLong(sortInfo1.getCustOrderCode().substring(3)) > Long.parseLong(sortInfo2.getCustOrderCode().substring(3))) {
                    return 1;
                }
                return -1;
            }).collect(Collectors.toList());
            log.info("castle-execute-after查询出的sortInfo数据:{}", sortInfoList);

            //3.当数量未达到圆整包装数时
            if (qty < roundQty) {
                //4.查看客户排序信息钟的最早的一条过点时间 + 生产组最晚生成时间是否已到达当前时间
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                sortInfo = sortInfoList.get(0);
                //最晚延迟的时间
                int latestCreationTime = Integer.parseInt(partProdGroup.getLatestCreationTime());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(sdf.parse(sortInfo.getMatchTime()));
                calendar.add(Calendar.MINUTE, latestCreationTime);
                Date laterTime = calendar.getTime();

                if (laterTime.after(new Date())) {
                    log.info("当前时间:{}没有达到最晚生成时间:{}", sdf.format(new Date()), sdf.format(laterTime));
                    return;
                } else {
                    //如果最早来的排序信息的多点时间已经超过最晚生成时间，则圆整数为当前排序信息对应的合计数
                    roundQty = qty;
                }
            }

            //批量获取生生产序号
            /**
             * 生产序号
             * 工厂-产线-9位流水号,跳号10
             */
            GenSerialNoModel prodSeqSerialNoModel = new GenSerialNoModel("MES_WORK_ORDER_SEQ_WU_HAN");
            prodSeqSerialNoModel.setProdLocation(partProdGroup.getWorkCenterCode());
            List<String> proSeqResultList = syncFuncService.syncSerialNo(prodSeqSerialNoModel.organizeCode(partProdGroup.getOrganizeCode()), "edi", partProdGroup.getOrganizeCode(), sortInfoList.size()).getResultList();

            log.info("castle-execute-顺序号:{}", proSeqResultList);
            //扣减qty 生成工单
            while (qty > 0) {
                //临时客户排序信息中的数量计数器
                sortInfo = sortInfoList.get(0);
                String seq = proSeqResultList.get(0);
                long productSeq = Long.parseLong(seq.replace("MES_WORK_ORDER_SEQ_WU_HAN", "").replace(partProdGroup.getWorkCenterCode(), ""));

                double custQty = sortInfo.getQty();
                //当客户排序信息中的数量大于剩余圆整数量要Break
                if (custQty > roundQty) {
                    log.info("单笔订单数量大于圆整数量01,车号:{}", sortInfo.getVinCode());
                    return;
                }

                //查找该零件是否在生产组中
                MesPartProdGroupDetail mesPartProdGroupDetail = prodGroupDetailMap.get(sortInfo.getCustPartNo());
                if (mesPartProdGroupDetail == null) {
                    log.info("生产组中未找到该零件:{}", sortInfo.getCustPartNo());
                    return;
                }
                //生产零件号
                if (mesPartProdGroupDetail.getProductPartNo() == null) {
                    mesPartProdGroupDetail.setProductPartNo(mesPartProdGroupDetail.getPartNo());
                }
                /**
                 * 2024/11/14
                 * 不带z的零件号是装配费
                 * 零件号带z是erp零件号 生产零件号
                 */
                MesPartSap mesPart = this.getMesPart(mesPartProdGroupDetail.getProductPartNo(), partProdGroup.getOrganizeCode());

                if (mesPart == null) {
                    log.info("{}工厂没有,零件:{}信息", partProdGroup.getOrganizeCode(), mesPartProdGroupDetail.getProductPartNo());
                    return;
                }

                mesCreateWorkOrder.createWorkOrder(partProdGroup, mesPartProdGroupDetail, mesPart, sortInfo,productSeq);

                qty -= custQty;
                sortInfoList.remove(sortInfo);
                proSeqResultList.remove(seq);
                // 更新排序信息 已完成
                custSoftInfoService.updateCustSortInfo(sortInfo);
            }


        } catch (Exception e) {
            log.error("按包装圆整出错:MesPackageRoundnessService：{}", e.getMessage());
        }
    }

    private MesPartSap getMesPart(String productPartNo, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productPartNo, "partNo", ddlPackBean);
        List<MesPartSap> parts = mesPartSapRao.findByHqlWhere(ddlPackBean);
        return parts.isEmpty() ? null : parts.get(0);
    }

}
